import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "KragenPlugs.KragenSlots";
import "KragenPlugs.Utils";
import "KragenPlugs.Utils.UI.DragBar";

KragenSlot = class( Turbine.UI.Window );

function KragenSlot:Constructor(index, name, x, y, properties)

	Turbine.UI.Window.Constructor( self );
	
	self.name = name;
	self.index = index;
	self.edit = false;
	
	-- get settings or set defaults if not set
	if (properties == nil) then
		self:Default(x, y);
	else
		self.settings = properties;
	end
	
	-- SET PROPERTIES
	self:SetSize(35, 35);
	self:SetPosition(ValidatePosition(self.settings.posX, self.settings.posY, self:GetWidth(), self:GetHeight()));
	self:SetVisible(true);
	self:SetZOrder(-10);
	
	-- define the quickslot
	self.blank = Turbine.UI.Lotro.Quickslot();
	self.blank:SetParent( self );
	self.blank:SetZOrder(-1);
	self.blank:SetSize( 35, 35 );
	self.blank:SetPosition( 0, 0 );
	self.blank:SetVisible(true);
	
	self.qswin = Turbine.UI.Window();
	self.qswin:SetParent( self );
	self.qswin:SetSize( 35, 35 );
	self.qswin:SetPosition( 0, 0 );
	self.qswin:SetVisible(true);
	self.qswin:SetZOrder(1);
	
	self.quickslot = Turbine.UI.Lotro.Quickslot();
	self.quickslot:SetParent( self.qswin );
	self.quickslot:SetSize( 35, 35 );
	self.quickslot:SetPosition( 0, 0 );
	self.quickslot:SetVisible(true);
	
	self.prev = Turbine.UI.Window();
	self.prev:SetParent(self);
	self.prev:SetVisible( false );
	self.prev:SetSize(35,35);
  	self.prev:SetPosition(0, -35);
  	self.prev:SetOpacity(self.settings.minopacity);
  	
  	self.prevqs = Turbine.UI.Lotro.Quickslot();
	self.prevqs:SetParent( self.prev );
	self.prevqs:SetSize( 35, 35 );
	self.prevqs:SetPosition( 0, 0 );
    self.prevqs:SetAllowDrop(false);
	self.prevqs:SetZOrder(1);
	
	self.next = Turbine.UI.Window();
	self.next:SetParent(self);
    self.next:SetVisible( false );
	self.next:SetSize(35,35);
  	self.next:SetPosition(0, 35);
  	self.next:SetOpacity(self.settings.minopacity);
  	
  	self.nextqs = Turbine.UI.Lotro.Quickslot();
	self.nextqs:SetParent( self.next );
	self.nextqs:SetSize( 35, 35 );
	self.nextqs:SetPosition( 0, 0 );
  	self.nextqs:SetAllowDrop(false);
	self.nextqs:SetZOrder(1);
	
	
	-- DRAG BAR!
	-- attach dragbar class to window
  	self.dragBar = KragenPlugs.Utils.UI.DragBar( self, self.name );
	self.dragBar:SetRotator(true);
	self.dragBar:SetRotatorOffset(2, 25);
    
	-- save position when user stops dragging window
	self.DragEnd = function(sender)
        local x, y = self:GetPosition();
        self.settings.posX = x;
        self.settings.posY = y;
		self:SetPosition(ValidatePosition(self.settings.posX, self.settings.posY, self:GetWidth(), self:GetHeight()));
		SaveSettings(self.name, self.settings);
		self:SetWantsUpdates(true);
    end
	
	-- Snap to Grid when user stops dragging
    self.Update = function( sender )
    	local x, y = self:GetPosition();
        
        if (settings.snaptogrid) then
	        local temp = x / settings.gridsize;
			if (temp - math.floor(temp) > .5) then
				x = settings.gridsize * math.ceil(temp);
			else
				x = settings.gridsize * math.floor(temp);
			end

	        local temp = y / settings.gridsize;
			if (temp - math.floor(temp) > .5) then
				y = settings.gridsize * math.ceil(temp);
			else
				y = settings.gridsize * math.floor(temp);
			end
		end

        self.settings.posX = x;
        self.settings.posY = y;
        
        self:SetPosition(x, y);
        self.dragBar:RecalculatePosition();
        
        self:SetWantsUpdates(false);
	end
    
    -- set visablity based on incombat and outcombat when HUD is turned on
    self.HUDShow = function(sender)
        if ((self.settings.active == 3 and localplayer:IsInCombat()) or (self.settings.active == 2 and not localplayer:IsInCombat())) then
	   		self:SetVisible( false );
		end
    end
	
	self.Rotate = function(sender, args)
		if (self.settings.preview == 2) then
			self:SetPreview(3);
		elseif (self.settings.preview == 3) then
			self:SetPreview(2);
		elseif (self.settings.preview == 4) then
			self:SetPreview(5);
		elseif (self.settings.preview == 5) then
			self:SetPreview(4);
		end
	end
	
	
	-- CALL METHODS!
	self:SetChangeOn(self.settings.changeon)
	self:SetActive(self.settings.active);
	self:SetPreview(self.settings.preview);
	self:CleanShortcuts();
	
	
	
	-- INCREMENT SLOT
	-- trigger quickslot change when you scroll the quickslot (if change on is set to both or scroll)
	self.quickslot.MouseWheel = function (sender,args) self.Scroll(args); end
	self.prevqs.MouseWheel = function (sender,args) self.Scroll(args); end
	self.nextqs.MouseWheel = function (sender,args) self.Scroll(args); end
	self.Scroll = function (args)
		if (self.blanknum ~= 0) then
			if (args.Direction * -1 == 1) then
				self.settings.currentshortcut = self.settings.currentshortcut - 1;
			end
			self:CleanShortcuts();
			self.blanknum = 0;
		end
		if (self.settings.changeon ~= 2) then
			self:IncrementShortcut(args.Direction * -1);
		end
	end
	
	-- trigger quickslot change when you click the quickslot (if change on is set to both or click)
	self.quickslot.MouseClick = function (sender,args)
		if (self.settings.changeon ~= 1) then
			if ( args.Button == Turbine.UI.MouseButton.Left ) then
				self.quickslot:SetWantsUpdates(true);
			else
				self:IncrementShortcut(1);
			end
		end
	end
	
	-- delay timer for click quickslot change
	self.quickslot.Update = function (sender, args)
		 self.quickslot:SetWantsUpdates(false);
		 self:IncrementShortcut(1);
	end
	
	
	-- DRAG AND DROP EVENTS
	-- add blank slot when skill/item is dragged into quickslot (if slot isnt locked and alt is down)
	self.quickslot.DragEnter = function (sender, args)
		if (not self.settings.locked) then
			if (self:IsAltKeyDown() and self.blanknum == 0) then
				self:AddBlank(self.settings.currentshortcut + 1);
				if (self.settings.preview ~= 1) then
					self.prev:SetVisible(true);
					self.next:SetVisible(true);
					self.prevqs:SetVisible(true);
					self.nextqs:SetVisible(true);
				end
			end
		end
	end
	
	-- remove skill/item from slot (if slot isnt locked)
	self.quickslot.DragLeave = function (sender, args)
		if (not self.settings.locked) then
			if (self.mousedown) then
				self.mousedown = false;
				self.settings.shortcut[self.settings.currentshortcut] = { Type = 0, Data = "" };
				self.settings.currentshortcut = self.settings.currentshortcut - 1;
			end
			self:CleanShortcuts();
			self.blanknum = 0;
		end
	end
	
	-- on drag drop, add item to currently showing spot
	self.quickslot.DragDrop = function (sender, args)
		if (not self.settings.locked) then
			if (self.settings.currentshortcut == 0) then
				self.settings.totalshortcuts = self.settings.totalshortcuts + 1;
				self.settings.shortcut[self.settings.totalshortcuts] = { Type = args.DragDropInfo:GetShortcut():GetType(), Data = args.DragDropInfo:GetShortcut():GetData() };
				self.settings.currentshortcut = self.settings.totalshortcuts;
				self.qswin:SetOpacity(self.settings.maxopacity);
				self:SetShortcut();
			elseif (self.settings.shortcut[self.settings.currentshortcut] == nil or self.settings.shortcut[self.settings.currentshortcut].Data == "") then
				self.settings.shortcut[self.settings.currentshortcut] = { Type = args.DragDropInfo:GetShortcut():GetType(), Data = args.DragDropInfo:GetShortcut():GetData() };
				self.qswin:SetOpacity(self.settings.maxopacity);
				self:SetShortcut();
				self.blanknum = 0;
			else
				self.settings.shortcut[self.settings.currentshortcut] = { Type = args.DragDropInfo:GetShortcut():GetType(), Data = args.DragDropInfo:GetShortcut():GetData() };
				self:SetShortcut();
			end
		else
			self:SetShortcut();
		end
	end
	
	
	-- MOUSE EVENTS
	-- set mouse down to true to facilitate dragging
	self.quickslot.MouseDown = function(sender, args)
		if (not self.settings.locked) then
			self.mousedown = true;
		end
	end
	
	-- trigger hover effect off when mouse leaves quickslot
	self.quickslot.MouseLeave = function (sender, args)
		if (self.settings.hover) then
		    self.qswin:SetOpacity(self.settings.minopacity);
		else
			self.qswin:SetOpacity(self.settings.maxopacity);
		end
		
		self.next:SetOpacity(self.settings.minopacity);
		self.prev:SetOpacity(self.settings.minopacity);
		
		if (self.settings.preview < 4) then
			self.prev:SetVisible(false);
			self.next:SetVisible(false);
		end
	end
	
	-- trigger hover effect on when mouse enters quickslot
	self.quickslot.MouseEnter = function (sender, args)
		if (self.settings.hover) then
		    self.qswin:SetOpacity(self.settings.maxopacity);
		end
		
		if (self.settings.preview ~= 1) then
			self.prev:SetVisible(true);
			self.next:SetVisible(true);
			self.prevqs:SetVisible(true);
			self.nextqs:SetVisible(true);
			self.next:SetOpacity(self.settings.minopacity);
			self.prev:SetOpacity(self.settings.minopacity);
		end
	end
	
	-- PLAYER EVENTS
	-- combat changed event
	AddCallback(localplayer, "InCombatChanged", function(sender, args)
		self:InCombatChanged();
	end);

end

-- Default Settings
function KragenSlot:Default(x, y)
    self.settings = { };
	self.settings.locked = false;
	self.settings.changeon = 1;
	self.settings.active = 1;
	self.settings.preview = 1;
	
	self.settings.hover = false;
	self.settings.minopacity = .5;
	self.settings.maxopacity = 1;
	self.settings.posX = x;
    self.settings.posY = y;
	
	self.settings.shortcut = { };
	self.settings.totalshortcuts = 1;
	self.settings.currentshortcut = 1;
end

-- Combat State Changed
function KragenSlot:InCombatChanged()
	if (not self.edit) then
		if (localplayer:IsInCombat()) then
			if ((self.settings.active == 2 or self.settings.active == 1) and not self:IsVisible()) then
				self:SetVisible( true );
			elseif ((self.settings.active == 3) and self:IsVisible()) then
				self:SetVisible( false );
			end
		else
			if ((self.settings.active == 3 or self.settings.active == 1) and not self:IsVisible()) then
				self:SetVisible( true );
			elseif ((self.settings.active == 2) and self:IsVisible()) then
				self:SetVisible( false );
			end
		end
	end
end

-- Increment Shortcut
function KragenSlot:IncrementShortcut(inc)
	self.settings.currentshortcut = self.settings.currentshortcut + inc;
	
	if (self.settings.currentshortcut > self.settings.totalshortcuts) then
	    self.settings.currentshortcut = 1;
	elseif (self.settings.currentshortcut == 0) then
	    self.settings.currentshortcut = self.settings.totalshortcuts;
	end
	
	self:SetShortcut()
end

-- Set Current Shortcut
function KragenSlot:SetShortcut()
	local prevnum = 0;
	local nextnum = 0;
	
	-- calculate previous shortcut with wrapping
	if (self.settings.currentshortcut - 1 > 0) then
		prevnum = self.settings.currentshortcut - 1;
	else
  		prevnum = self.settings.totalshortcuts;
	end

	-- calculate next shortcut with wrapping
	if (self.settings.currentshortcut + 1 > self.settings.totalshortcuts) then
  		nextnum = 1;
	else
  		nextnum = self.settings.currentshortcut + 1;
	end

	-- set main shortcut
	if not pcall(function() self.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[self.settings.currentshortcut].Type, self.settings.shortcut[self.settings.currentshortcut].Data)); end) then
		
	end

	-- set prev shortcut
	if not pcall(function() self.prevqs:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[prevnum].Type, self.settings.shortcut[prevnum].Data)); end) then
        
	end
	
	-- set next shortcut
	if not pcall(function() self.nextqs:SetShortcut(Turbine.UI.Lotro.Shortcut(self.settings.shortcut[nextnum].Type, self.settings.shortcut[nextnum].Data)); end) then
        
	end
	
	-- set main opacity
	if (self.settings.shortcut[self.settings.currentshortcut] == nil or self.settings.shortcut[self.settings.currentshortcut].Type == 0) then
		self.qswin:SetOpacity(0);
	else
		if (self.settings.hover) then
			self.qswin:SetOpacity(self.settings.minopacity);
		else
			self.qswin:SetOpacity(self.settings.maxopacity);
		end
	end
	
	-- set next opacity
	if (self.settings.shortcut[nextnum] == nil or self.settings.shortcut[nextnum].Type == 0) then
		self.next:SetOpacity(0);
	else
		self.next:SetOpacity(self.settings.minopacity);
	end
	
	-- set prev opacity
	if (self.settings.shortcut[prevnum] == nil or self.settings.shortcut[prevnum].Type == 0) then
		self.prev:SetOpacity(0);
	else
		self.prev:SetOpacity(self.settings.minopacity);
	end
end

-- clean up the shortcuts and remove spaces
function KragenSlot:CleanShortcuts()
	local newshortcuts = { };
	local c = 1;
	for i = 1, self.settings.totalshortcuts do
		if (self.settings.shortcut[i] ~= nil and self.settings.shortcut[i].Type ~= 0) then
			newshortcuts[c] = self.settings.shortcut[i];
			c = c + 1;
		end
	end
	if (self.settings.currentshortcut == self.blanknum) then
		self.settings.currentshortcut = self.settings.currentshortcut - 1;
	end
	self.settings.totalshortcuts = #newshortcuts;
	self.settings.shortcut = newshortcuts;
	
	if (self.settings.totalshortcuts == 0) then
		self.settings.totalshortcuts = 1;
		self.settings.currentshortcut = 1;
		self.blanknum = 1;
		self.settings.shortcut[1] = {Type=0, Data=""};
	end
	self:SetShortcut();
	
end

-- Add blank quickslot to list
function KragenSlot:AddBlank(num)
	self.settings.totalshortcuts = self.settings.totalshortcuts + 1;
	local newshortcuts = { };
	for k,v in pairs(self.settings.shortcut) do
		if (k >= num) then
			newshortcuts[k+1] = v;
		else
			newshortcuts[k] = v;
		end
	end
	
	newshortcuts[num] = {Type = 0, Data = ""};
	self.settings.currentshortcut = num;
	self.settings.shortcut = newshortcuts;
	self:SetShortcut();
	
	self.blanknum = num;
end


-- Set preview mode
function KragenSlot:SetPreview(preview)
	if (preview == 4 or preview == 5) then
		self.prev:SetVisible(true);
		self.next:SetVisible(true);
		self.prevqs:SetVisible(true);
		self.nextqs:SetVisible(true);
	else
		self.prev:SetVisible(false);
		self.next:SetVisible(false);
	end
	if (preview == 1) then
		if (self.settings.preview == 4 or self.settings.preview == 2) then
			self:SetPosition(self:GetLeft()+35, self:GetTop());
		elseif (self.settings.preview == 5 or self.settings.preview == 3) then
			self:SetPosition(self:GetLeft(), self:GetTop()+35);
		end
		self.qswin:SetPosition(0,0);
		self.blank:SetPosition(0,0);
		self:SetSize(35, 35);
	elseif (preview == 4 or preview == 2) then
		if (self.settings.preview == 1) then
			self:SetPosition(self:GetLeft()-35, self:GetTop());
		elseif (self.settings.preview == 3 or self.settings.preview == 5) then
			self:SetPosition(self:GetLeft()-35, self:GetTop()+35);
		end
		self.qswin:SetPosition(35,0);
		self.blank:SetPosition(35,0);
		self.prev:SetPosition(0, 3);
		self.next:SetPosition(70, 3);
		self:SetSize(105, 38);
	elseif (preview == 5 or preview == 3) then
		if (self.settings.preview == 1) then
			self:SetPosition(self:GetLeft(), self:GetTop()-35);
		elseif (self.settings.preview == 4 or self.settings.preview == 2) then
			self:SetPosition(self:GetLeft()+35, self:GetTop()-35);
		end
		self.qswin:SetPosition(0,35);
		self.blank:SetPosition(0,35);
		self.prev:SetPosition(3, 0);
		self.next:SetPosition(3, 70);
		self:SetSize(38, 105);
	end
	
	self.settings.posX = self:GetLeft();
	self.settings.posY = self:GetTop();
	self:SetPosition(ValidatePosition(self.settings.posX, self.settings.posY, self:GetWidth(), self:GetHeight()));
	self.dragBar:Refresh();
	self.settings.preview = preview;
	SaveSettings();
	
	if (setup ~= nil and setup.setupSlots.slotlist:GetSelection() == self.index) then
		if (setup.setupSlots.preview.combobox:GetSelection() ~= preview) then
			setup.setupSlots.preview.combobox:SetSelection(preview);
		end
	end
end

function KragenSlot:SetIndex(index)
	self.index = index;
end

function KragenSlot:SetChangeOn(changeon)
	self.settings.changeon = changeon;
	-- set right click off if quickslot is set to change on click
	if (self.settings.changeon ~= 1) then
        self.quickslot:SetUseOnRightClick(false);
	else
	    self.quickslot:SetUseOnRightClick(true);
	end
end

function KragenSlot:SetActive(active)
	self.settings.active = active;
	self:InCombatChanged();
end

function KragenSlot:SetEdit(bol)
	self.edit = bol;
	if (bol) then
		self:SetBackColor(Turbine.UI.Color( 0, 0, 1 ));
		self:SetVisible(true);
	else
		self:SetBackColor(Turbine.UI.Color( nil ));
		self:InCombatChanged();
	end
end
