import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "KragenPlugs.KragenSlots";
import "KragenPlugs.Utils";

-- screen width and height for default quickslot position
local displayWidth = Turbine.UI.Display:GetWidth();
local displayHeight = Turbine.UI.Display:GetHeight();

-- Hook into player for access to combat status
localplayer = Turbine.Gameplay.LocalPlayer:GetInstance();

-- Get Client Locale
locale = Turbine.Engine.GetLocale();

-- table for holding instances of KragenSlot
slots = { };

-- load and iniate saved KragenSlots
function LoadSettings()
	settings = DecodeObject(Turbine.PluginData.Load( Turbine.DataScope.Character, "KragenSlotsSettings" ));
	if (settings == nil) then
		settings = { };
		settings.slots = { };
		settings.totalslots = 0;
		settings.gridsize = 36;
		settings.snaptogrid = false;
		settings.KSButton = { };
		settings.KSButton.x = 10;
		settings.KSButton.y = 10;
		settings.KSButton.active = true;
		Turbine.Shell.WriteLine("Welcome to KragenSlots. \n To create new KragenSlots type /kragenslots setup to access the setup menu.  Hold Alt while dragging a skill/item into a slot to add it as a blank space.");
	else
		-- Upgrade from v1
		if (settings.quickslotinfo ~= nil) then
			for k,v in pairs(settings.quickslotinfo) do
				if (v.onscroll and v.onclick) then
					v.changeon = 3;
				elseif (v.onscroll) then
					v.changeon = 1;
				elseif (v.onclick) then
					v.changeon = 2;
				end
				if (v.incombat and v.outcombat) then
					v.active = 1;
				elseif (v.incombat) then
					v.active = 2;
				elseif (v.outcombat) then
					v.active = 3;
				end
				if (v.preview) then
					v.preview = 3;
				else
					v.preview = 1;
				end
				v.locked = false;
				v.onscroll = nil;
				v.onclick = nil;
				v.incombat = nil;
				v.outcombat = nil;
			end
			settings.slots = { };
			local cs = 1;
			for k,v in pairs(settings.quickslotinfo) do
				settings.slots[cs] = v;
				settings.slots[cs].name = k;
				cs = cs + 1;
			end
			settings.gridsize = 36;
			settings.snaptogrid = false;
			settings.KSButton = { };
			settings.KSButton.x = 10;
			settings.KSButton.y = 10;
			settings.KSButton.active = true;
		end
		
		-- initalize slots
		local c = 1;
	    for k,v in pairs(settings.slots) do
			slots[c] = KragenSlot(c, settings.slots[k].name, 1, 1, settings.slots[k]);
			c = c + 1;
		end
		settings.totalslots = c - 1;
	end
end

-- function to save a quickslots info (called from KragenSlot class)
function SaveSettings()
	for k,v in pairs(slots) do
		settings.slots[k] = v.settings;
		settings.slots[k].name = v.name;
	end
	settings.quickslotinfo = nil;
	Turbine.PluginData.Save( Turbine.DataScope.Character, "KragenSlotsSettings", EncodeObject(settings) );
end

-- on unload call each instances unload function to save settings
function UnloadPlugin()
	SaveSettings();
end

-- add callback function
function AddCallback(object, event, callback)
    if (object[event] == nil) then
            object[event] = callback;
    else
            if (type(object[event]) == "table") then
                    table.insert(object[event], callback);
            else
                    object[event] = {object[event], callback};
            end
    end
end


-- Validate window Position
function ValidatePosition(x, y, width, height)
    x = math.min(math.max(x, 0), Turbine.UI.Display:GetWidth() - width);
    y = math.min(math.max(y, 0), Turbine.UI.Display:GetHeight() - height);
    return x, y;
        
end

function EncodeObject(obj)
	if ((locale == "de" or locale == "fr") and obj ~= nil) then
		local encoded = {};
		local function _copyObject(obj)
			local function _encodeObject(obj)
				if (type(obj) == "number") then return (("<num>%f</num>"):format(obj)); else return obj end;
			end
			if (type(obj) ~= "table") then 
				return _encodeObject(obj);
			elseif (encoded[obj] ~= nil) then
				return encoded[obj];
			end
			local newtable = { };
			for k,v in pairs(obj) do
				local key = _copyObject(k);
				local value = _copyObject(v);
				newtable[key] = value;
				encoded[obj] = newtable[key];
			end
			return newtable;
		end
		
		return _copyObject(obj);
	else
		return obj;
	end
end

function DecodeObject(data)
	if ((locale == "de" or locale == "fr") and data ~= nil) then
		local decoded = { };
		local function _copyObject(obj)
			local function _decodeObject(obj)
				if (type(obj) == "string") then 
					local match = obj:match("<num>(%d+.%d+)</num>");
					if (match ~= nil) then
						return tonumber(match);
					else
						return obj
					end
				else
					return obj;
				end
			end
			if (obj == nil) then return nil; end
			if (type(obj) ~= "table") then
				return _decodeObject(obj);
			elseif (decoded[obj]) then
				return decoded[obj];
			end
			local newtable = { };
			decoded[obj] = newtable;
			for k,v in pairs(obj) do
				local key = _copyObject(k);
				local value = _copyObject(v);
				newtable[key] = value;
				decoded[obj] = newtable[key];
			end
			return newtable;
		end
		
		return _copyObject(data);
	else
		return data;
	end
end



-- setup chat commands
Command = Turbine.ShellCommand();
function Command:Execute( command, arguments )
	if (arguments == "setup") then
		setup:SetVisible(true);
	else
		Turbine.Shell.WriteLine("Welcome to KragenSlots. \n To create new KragenSlots type /kragenslots setup to access the setup menu.  Hold Alt while dragging a skill/item into a slot to add it to the rotation.");
	end

end



-- inialize plugin
function init()
	-- setup plugin unload function to save setting on unload.
	pluginName="KragenSlots";
	window = Turbine.UI.Window();
	window:SetWantsUpdates(true);
	window.Update = function(sender,args)
		if (Plugins[pluginName] ~= nil) then
			Plugins[pluginName].Unload = function(self,sender,args)
				UnloadPlugin();
			end
			window:SetWantsUpdates(false);
		end
	end
	
	-- add chat commands to turbine shell
	Turbine.Shell.AddCommand( "kragenslots", Command );
	
	-- load settings
	LoadSettings();
	
	-- initialize setup window
	setup = SetupWindow();
	
	-- initialize KSButton
	menu = KSButton(settings.KSButton.x, settings.KSButton.y);
	menu:SetVisible(settings.KSButton.active);
	setup.setupGeneral.ksbutton:SetChecked(settings.KSButton.active);
	
	-- Populate setup window list
	local locked = true;
	setup.setupSlots.slotlist:AddItem("", 0);
	for k,v in pairs(settings.slots) do
		setup.setupSlots.slotlist:AddItem(v.name, k);
		if (not v.locked) then
			locked = false;
		end
	end
	
	-- Set Locked Status
	if (locked) then
		setup.setupGeneral.buttonlock:SetText("Unlock All");
	else
		setup.setupGeneral.buttonlock:SetText("Lock All");
	end
	
	-- Set Grid Settings
	setup.setupGeneral.snaptogrid:SetChecked(settings.snaptogrid);
	setup.setupGeneral.gridsize:SetValue(settings.gridsize);
	
	-- Setup Window Events
	-- General: Show KS Icon Changed
	setup.setupGeneral.ksbutton.CheckedChanged = function(sender, args)
		settings.KSButton.active = setup.setupGeneral.ksbutton:IsChecked();
		menu:SetVisible(setup.setupGeneral.ksbutton:IsChecked());
	end
	
	
	-- General: Lock Button Clicked
	setup.setupGeneral.buttonlock.Click = function(sender, args)
		local locked = (setup.setupGeneral.buttonlock:GetText() == "Lock All");
		for k,v in pairs(slots) do
			v.settings.locked = locked;
		end
		if (setup.setupSlots.slotlist:GetSelection() ~= 0 and setup.setupSlots.slotlist:GetSelection() ~= nil and setup.setupSlots.lockslot:IsChecked() ~= locked) then
			setup.setupSlots.lockslot:SetChecked(locked);
		end
		
		if (locked) then
			setup.setupGeneral.buttonlock:SetText("Unlock All");
		else
			setup.setupGeneral.buttonlock:SetText("Lock All");
		end
	end
	
	-- General: Snap to Grid Changed
	setup.setupGeneral.snaptogrid.CheckedChanged = function (sender, args)
		settings.snaptogrid = setup.setupGeneral.snaptogrid:IsChecked();
	end
	
	-- General: Grid Size Changed
	setup.setupGeneral.gridsize.ValueChanged = function (sender, args)
		settings.gridsize = setup.setupGeneral.gridsize:GetValue();
	end
	
	-- Slots: New Slot
	setup.setupSlots.slotbuttonnew.Click = function(sender, args)
		settings.totalslots = settings.totalslots + 1;
		slots[settings.totalslots] = KragenSlot(settings.totalslots, "New Slot", settings.totalslots * 36 + 600, displayHeight * 1 / 3);
		setup.setupSlots.slotlist:AddItem("New Slot", settings.totalslots);
		setup.setupSlots.slotlist:SetSelection(settings.totalslots);
	end
	
	-- Slots: Remove Slot
	setup.setupSlots.slotbuttonremove.Click = function(sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()].dragBar:SetVisible(false);
			slots[setup.setupSlots.slotlist:GetSelection()]:SetVisible(false);
			slots[setup.setupSlots.slotlist:GetSelection()] = nil;
			settings.slots[setup.setupSlots.slotlist:GetSelection()] = nil;
			setup.setupSlots.slotlist:SetSelection(0);
			setup.setupSlots.slotnamebox:SetWantsUpdates(false);
			setup.setupSlots.slotlist.listBox:ClearItems();
			setup.setupSlots.slotlist:AddItem("", 0);
			local c = 1;
			for i=1, settings.totalslots do
				if (slots[i] ~= nil) then
					settings.slots[c] = settings.slots[i];
					slots[c] = slots[i];
					slots[c]:SetIndex(c);
					c = c + 1;
				else
					settings.slots[i] = nil;
				end
			end
			settings.totalslots = c - 1;
			for k,v in pairs(settings.slots) do
				setup.setupSlots.slotlist:AddItem(v.name, k);
				if (not v.locked) then
					locked = false;
				end
			end
			
			if (locked) then
				setup.setupGeneral.buttonlock:SetText("Unlock All");
			else
				setup.setupGeneral.buttonlock:SetText("Lock All");
			end
		
			setup.setupSlots.slotnamebox:SetWantsUpdates(true);
			SaveSettings();
		end
	end
	
	setup.SetupClosing = function (sender, args)
		for k,v in pairs(slots) do
			v:SetEdit(false);
		end
		setup.setupSlots.slotlist:SetSelection(0); 
	end
	
	-- Slots: Selected Slot Changed
	setup.setupSlots.slotlist.SelectedIndexChanged = function (sender, args)
		for k,v in pairs(slots) do
			v:SetEdit(false);
		end
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			setup.setupSlots.slotnamebox:SetText(slots[setup.setupSlots.slotlist:GetSelection()].name);
			setup.setupSlots.lockslot:SetChecked(slots[setup.setupSlots.slotlist:GetSelection()].settings.locked);
			setup.setupSlots.changeon.combobox:SetSelection(slots[setup.setupSlots.slotlist:GetSelection()].settings.changeon);
			setup.setupSlots.active.combobox:SetSelection(slots[setup.setupSlots.slotlist:GetSelection()].settings.active);
			setup.setupSlots.preview.combobox:SetSelection(slots[setup.setupSlots.slotlist:GetSelection()].settings.preview);
			setup.setupSlots.hover:SetChecked(slots[setup.setupSlots.slotlist:GetSelection()].settings.hover);
			setup.setupSlots.minopacity:SetValue(slots[setup.setupSlots.slotlist:GetSelection()].settings.minopacity * 100);
			setup.setupSlots.maxopacity:SetValue(slots[setup.setupSlots.slotlist:GetSelection()].settings.maxopacity * 100);
			setup.setupSlots.slotnamebox:SetVisible(true);
			setup.setupSlots.slotbuttonremove:SetVisible(true);
			setup.setupSlots.lockslot:SetVisible(true);
			setup.setupSlots.changeon.label:SetVisible(true);
			setup.setupSlots.changeon.combobox:SetVisible(true);
			setup.setupSlots.active.label:SetVisible(true);
			setup.setupSlots.active.combobox:SetVisible(true);
			setup.setupSlots.preview.label:SetVisible(true);
			setup.setupSlots.preview.combobox:SetVisible(true);
			setup.setupSlots.hover:SetVisible(true);
			setup.setupSlots.minopacity:SetVisible(true);
			setup.setupSlots.maxopacity:SetVisible(true);
			slots[setup.setupSlots.slotlist:GetSelection()]:SetEdit(true);
		else
			setup.setupSlots.slotnamebox:SetVisible(false);
			setup.setupSlots.slotbuttonremove:SetVisible(false);
			setup.setupSlots.lockslot:SetVisible(false);
			setup.setupSlots.changeon.label:SetVisible(false);
			setup.setupSlots.changeon.combobox:SetVisible(false);
			setup.setupSlots.active.label:SetVisible(false);
			setup.setupSlots.active.combobox:SetVisible(false);
			setup.setupSlots.preview.label:SetVisible(false);
			setup.setupSlots.preview.combobox:SetVisible(false);
			setup.setupSlots.hover:SetVisible(false);
			setup.setupSlots.minopacity:SetVisible(false);
			setup.setupSlots.maxopacity:SetVisible(false);
		end
	end
	
	-- Slots: Lock Changed
	setup.setupSlots.lockslot.CheckedChanged = function (sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()].settings.locked = setup.setupSlots.lockslot:IsChecked();
		end
		
		local locked
		
		for k,v in pairs(slots) do
			if (not v.locked) then
				locked = false;
			end
		end
		
		if (locked) then
			setup.setupGeneral.buttonlock:SetText("Unlock All");
		else
			setup.setupGeneral.buttonlock:SetText("Lock All");
		end
	end
	
	-- Slots: Change On Changed
	setup.setupSlots.changeon.combobox.SelectedIndexChanged = function (sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()]:SetChangeOn(setup.setupSlots.changeon.combobox:GetSelection());
		end
	end
	
	-- Slots: Active Changed
	setup.setupSlots.active.combobox.SelectedIndexChanged = function (sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()]:SetActive(setup.setupSlots.active.combobox:GetSelection());
		end
	end
	
	-- Slots: Preview Mode Changed
	setup.setupSlots.preview.combobox.SelectedIndexChanged = function (sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()]:SetPreview(setup.setupSlots.preview.combobox:GetSelection());
		end
	end
	
	-- Slots: Hover Enabled Changed
	setup.setupSlots.hover.CheckedChanged = function (sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()].settings.hover = setup.setupSlots.hover:IsChecked();
			slots[setup.setupSlots.slotlist:GetSelection()].quickslot.MouseLeave();
		end
	end
	
	-- Slots: Minimum Opacity Slider Changed
	setup.setupSlots.minopacity.ValueChanged = function (sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()].settings.minopacity = setup.setupSlots.minopacity:GetValue()/100;
			slots[setup.setupSlots.slotlist:GetSelection()].quickslot.MouseLeave();
		end
	end
	
	-- Slots: Maximum Opacity Slider Changed
	setup.setupSlots.maxopacity.ValueChanged = function (sender, args)
		if (setup.setupSlots.slotlist:GetSelection() ~= 0) then
			slots[setup.setupSlots.slotlist:GetSelection()].settings.maxopacity = setup.setupSlots.maxopacity:GetValue()/100;
			slots[setup.setupSlots.slotlist:GetSelection()].quickslot.MouseLeave();
		end
	end
	
	-- Display Setup on load if no slots exsist
	if (settings.totalslots == 0) then
		setup:SetVisible(true);
	end
end


init();